'  ---------------------------------------------------------------------------
'  SIMPLE SOCCER SPRITE DEMO
'  By Damian Rajkowski (WierdPumpkin)
'  15/12/2004

'  This source code can be freely distributed by any means.

'  THIS SOURCE CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND. USE AT YOUR OWN RISK.

'  This simple example demonstrates the following capabilities of the sprite system:

'       1. Loading sprites with .PNG And .JPG images. The .PNG format provides compression And an alpha channel For
'          masking out unwanted areas of the sprite (see point 5).
'       2. Using .X objects as templates - unlimited sprites with only a few .x objects!
'       3. Texture Swapping - For unlimited sprites.
'       4. Resizing/Stretching.
'       5. Masking - using the Alpha Layer To create irregular sprites.
'       6. Blending - To achieve special effects like shadowing And specular highlights.
'       7. Rotation.

'  ---------------------------------------------------------------------------
'  SYNOPSIS

'  This demo displays a falling soccerball, And a user-controlled paddle (Left/Right arrow keys) which can be used
'  To 'bounce' the soccer ball.

'  After compilation, this source code is turned into a file named 3impactd.dll And saved To the
'  3impactWork folder on your hard disk.

'  ---------------------------------------------------------------------------
'  TEMPLATE OBJECTS

'  This demo uses the Object files 'sprite.x' and 'screen.x' as templates for all sprites in the demo.
'  'sprite.x' is about 1x1 units, and 'screen.x' is about 12x12.

'  ---------------------------------------------------------------------------

Strict
Framework _3Impact.Lib

'  Constants To make our lives easier
Const ScreenWidth:Float   = 32.0
Const ScreenHeight:Float  = 24.0
Const ScreenCentreX:Float = 16.0
Const ScreenCentreY:Float = 12.0

'  Required 3Impact objects
Global Camera:CAMERA
Global Screen:SPRITE, Controls:SPRITE, Title:SPRITE, TitleShadow:SPRITE

'  To make vectors simpler, this derived class shortens the terious D3DXVECTOR2 name To VEC2, And provides a nifty
'  Set() Function To quickly populate the vector.
Type VEC2 Extends D3DXVECTOR2

    Method Set(X:Float, Y:Float)
       Self.x = X
	  Self.y = Y
    EndMethod

    Method SetFromVEC2(v:VEC2)
	  Self.x = v.x
	  Self.y = v.y 
    EndMethod
EndType

Function _VEC2:VEC2(X:Float = 0.0, Y:Float = 0.0)
     Local retval:VEC2=New VEC2
	retval.x = X 
	retval.y = Y 
	Return retval
EndFunction

'  A macro Function To simplify sprite positioning.
'  NOTE: The sprite coordinate system ranges from (-16,12) [Top-Left] To (16,-12) [bottom Right]. To make things
'        simpler (in my mind) i convert the sprite coordinates To a top-Left orientation: (0,0) To (32,24).
Function SetSpritePos(sprite:SPRITE, X:Float, Y:Float)
    iSpriteLocationSet(sprite,_VEC2(X - ScreenCentreX, -Y + ScreenCentreY))
EndFunction
Function SetSpritePosFromVEC2(sprite:SPRITE, Pos:VEC2)
    iSpriteLocationSet(sprite,_VEC2(Pos.x - ScreenCentreX, -Pos.y + ScreenCentreY))
EndFunction

'  A macro Function To simplify sprite loading
Function LoadSprite:SPRITE(_object:String, texture:String, Width:Float, Height:Float) 

    Local sprite:SPRITE

    '  Load specified template object. Don't affix to any specific camera by specifying NULL as second parameter.
    sprite = iSpriteCreate(_object,Null)

    '  Load actual sprite texture And swap it with the SPRITE* Object's original texture.
    '  This allows us To have unlimited unique sprites, using only a few .x objects.
    Local Image:TEXTURE
    Image = iTextureCreate(texture)
    iSpriteTextureSwap(sprite,Image)

    '  Scale the sprite To the specified dimensions. By basing your scale on the .x template, you can change the scale
    '  of ALL your sprites just by changing the template's size.
    iSpriteScaleSet(sprite,_VEC2(Width,Height))

    '  Return handle To New sprite
    Return sprite
EndFunction

'  This structure handles the user-controllable paddle
Type PADDLE
    Field decal:SPRITE, shadow:SPRITE
    Field pos:VEC2=_VEC2()

    '  Initialise all paddle sprites
    Method Initialise()

        '  First load the shadow image, so it appears behind the color image.
        '  NOTE: Because this shadow image will use blending, it MUST have white in any area where the image is supposed
        '        To be transparent, because the blending operation ignores the actual alpha layer therefore, To achieve
        '        masking, use white (value 1.0) which the blending operation below understands as 'no change'.
        shadow = LoadSprite("simplesoccer_res\\sprite.x","simplesoccer_res\\paddle_shadow.png",4.0,1.0)

        '  Set the shadow's blend mode to multiply the destination by the color in the shadow image, i.e. where the shadow
        '  image is light the background will show through in its natural color. Where its dark, the background will be
        '  darkened To give the impression of shadow.
        iSpriteRenderModeSet(shadow,D3DBLEND_DESTCOLOR,D3DBLEND_ZERO)

        '  Now load decal (color) image.
        decal = LoadSprite("simplesoccer_res\\sprite.x","simplesoccer_res\\paddle.png",4.0,1.0)

        '  Default the paddle To centre-bottom screen.
        pos.Set(ScreenCentreX, ScreenHeight - 2.0)

        '  Show sprites
        iSpriteShow(shadow)
        iSpriteShow(decal)
    EndMethod

    '  Automate paddle
    Method Run()

        '  If the user presses the Left/Right arrow keys, move the paddle within the screen boundaries
        If (iKeyDown(DIK_LEFT ) And pos.x >               5.0)
		 pos.x :- 0.3
	   ElseIf (iKeyDown(DIK_RIGHT) And pos.x < ScreenWidth - 5.0)
		 pos.x :+ 0.3
	   EndIf

        '  Set New position For decal And shadow sprites
        SetSpritePos(decal, pos.x,pos.y)
        SetSpritePos(shadow,pos.x + 0.4, pos.y + 0.4)

    EndMethod

EndType

'  This structure handles the automated ball, which has rudimentary physics.
Type BALL
    Field decal:SPRITE, selfshadow:SPRITE, shadow:SPRITE, specular:SPRITE
    Field pos:VEC2=_VEC2(), vel:VEC2=_VEC2()
    Field angle:Float, rotation:Float

    '  Initialise sprites
    Method Initialise()

        '  Load ball's shadow on ground first - so it falls behind all other ball images
        '  NOTE: Because this shadow image will use blending, it MUST have white in any area where the image is supposed
        '        To be transparent, because the blending operation ignores the actual alpha layer therefore, To achieve
        '        masking, use white (value 1.0) which the blending operation below understands as 'no change'.
        shadow = LoadSprite("simplesoccer_res\\sprite.x","simplesoccer_res\\ball_shadow.png",2.0,2.0)
        
        '  Set the shadow's blend mode to multiply the destination by the color in the shadow image, i.e. where the shadow
        '  image is light the background will show through in its natural color. Where its dark, the background will be
        '  darkened To give the impression of shadow.
        iSpriteRenderModeSet(shadow,D3DBLEND_DESTCOLOR,D3DBLEND_ZERO)

        '  Next, load the ball's image.
        decal = LoadSprite("simplesoccer_res\\sprite.x","simplesoccer_res\\ball.png",2.0,2.0)

        '  Now, load the Self-shadowing - which will be applied OVER the ball's image.
        '  NOTE: Because this shadow image will use blending, it MUST have white in any area where the image is supposed
        '        To be transparent, because the blending operation ignores the actual alpha layer therefore, To achieve
        '        masking, use white (value 1.0) which the blending operation below understands as 'no change'.
        selfshadow = LoadSprite("simplesoccer_res\\sprite.x","simplesoccer_res\\ball_selfshadow.png",2.0,2.0)

        '  Set the shadow's blend mode to multiply the destination by the color in the shadow image, i.e. where the shadow
        '  image is light the background will show through in its natural color. Where its dark, the background will be
        '  darkened To give the impression of shadow.
        iSpriteRenderModeSet(selfshadow,D3DBLEND_DESTCOLOR,D3DBLEND_ZERO)

        '  Finally, load the specular image - which will be applied OVER the
        specular = LoadSprite("simplesoccer_res\\sprite.x","simplesoccer_res\\ball_spec.png",2.0,2.0)

        '  Set the specular's blend mode to add the image color to the destination color, which has the effect of
        '  lightening the image (making it brighter). Since white is the maximum color value (1.0), adding white To
        '  any part of the image (as in the specular light spot) creates a shiny whiteness there.
        '  Adding black To any part of the image leaves it unchanged.
        iSpriteRenderModeSet(specular,D3DBLEND_ONE,D3DBLEND_ONE)

        '  Set attributes of position, velocity, rotation angle And angular velocity.
        pos.Set(ScreenCentreX, ScreenCentreY)
        vel.Set(0.1,0.0)
        angle = 0.0
        rotation = 3.0

        '  Display sprites
        iSpriteShow(shadow)
        iSpriteShow(decal)
        iSpriteShow(selfshadow)
        iSpriteShow(specular)
        
    EndMethod

    '  Automate ball
    Method Run() 

        '  Apply gravity To ball's vertical velocity
        vel.y :+ 0.01

        '  Apply current velocity To ball's position
        pos.PlusEq(vel)

        '  Check For collision with sides of screen
        If (pos.x < 2.0)

            '  Ball has collided with Left side - reverse ball's horizontal direction, and throw in some friction.
            vel.x = Abs(vel.x * 0.9)

            '  This may be a little confusing at first, but what this line is doing is ensuring the ball's 'spin'
            '  appears more natural. Throw a spinning ball against a wall, And its spin will be changed To lie
            '  more Or less in the direction of travel. This is doing something like that.
            '  Try commenting it out And see what happens.
            If ((rotation < 0.0 And vel.y > 0.0) Or (rotation > 0.0 And vel.y < 0.0)) rotation = -rotation

        ElseIf (pos.x > ScreenWidth - 2.0) 

            '  Ball has collided with Left side - reverse ball's horizontal direction, and throw in some friction.
            vel.x = -Abs(vel.x * 0.9)

            '  This may be a little confusing at first, but what this line is doing is ensuring the ball's 'spin'
            '  appears more natural. Throw a spinning ball against a wall, And its spin will be changed To lie
            '  more Or less in the direction of travel. This is doing something like that.
            '  Try commenting it out And see what happens.
            If ((rotation < 0.0 And vel.y < 0.0) Or (rotation > 0.0 And vel.y > 0.0)) rotation = -rotation

        EndIf

        '  Check For ball's collision with paddle, or fall from the bottom of the screen
        If (pos.y > paddle1.pos.y - 2.0) 

            If (pos.x > paddle1.pos.x - 5.0 And pos.x < paddle1.pos.x + 5.0) 
                '  Ball has collided with paddle.

                '  Adjust ball's position so it isn't protruding through paddle
                pos.y = paddle1.pos.y - 2.0

                '  Determine where on paddle ball has struck.
                If (pos.x < paddle1.pos.x - 3.0) 
                    '  Ball has struck Left rounded edge of paddle - rebound ball at an angle To create
                    '  illusion of realism. No pixel-perfect edge collisions here.
                    vel.x = -Abs(vel.y * 0.5)
                    vel.y = -vel.y

                ElseIf (pos.x > paddle1.pos.x + 3.0) 
                    '  Ball has struck Right rounded edge of paddle - rebound ball at an angle To create
                    '  illusion of realism. No pixel-perfect edge collisions here.
                    vel.x = Abs(vel.y * 0.5)
                    vel.y = -vel.y

                Else
                    '  Ball has struck paddle in the middle - do a simple rebound with a bit of friction.
                    vel.y = -vel.y * 0.9
                    vel.x :* 0.85

                    '  Check ball's resulting velocity, since friction will wear it down.
                    If (vel.y > -0.3) 
                        '  If ball is getting too slow, give it a kick in a random direction just For fun.
                        vel.y = -0.5
                        vel.x = iFloatRand(-0.5,0.5)
                    EndIf
                EndIf

            ElseIf (pos.y > ScreenHeight + 2.0)
                '  Ball has missed paddle And has fallen off the bottom of the screen - reset its position.
                pos.y = -2.0
                vel.y = 0.0
            EndIf

        EndIf

        '  Update angle with current rotation
        angle :- rotation

        '  Only set the angle of the ball's color image, the other images (specular/shadow) are light-based and
        '  should Not be affected by the ball's orientation (at least, not in this demo).
        iSpriteOrientationSet(decal,angle)

        '  Set positions of ball's color, self-shadow and specular images.
        SetSpritePosFromVEC2(decal,pos)
        SetSpritePosFromVEC2(selfshadow,pos)
        SetSpritePosFromVEC2(specular,pos)

        '  Set position of ball's shadow at slight offset, to create 3D feeling
        SetSpritePos(shadow,pos.x + 0.5,pos.y + 0.5)

    EndMethod

EndType

Global ball1:BALL=New BALL
Global paddle1:PADDLE=New PADDLE

If Not i3ImpactOpen() RuntimeError "Unable to initialize the 3Impact game engine"
If Not i3ImpactSettings(_ISETTINGS(640,480,32,False,True,False)) RuntimeError "Settings failed"

Local engine:sample_3IEngine=New sample_3IEngine

engine.Start()

Type sample_3IEngine Extends _3IEngine

	Method _Init()

	    '  First, we need a camera Object To render To, Or nothing can be displayed.
	    '  Here we create a Default camera which spans the entire screen.
	    Camera = iCameraCreate(0.0,0.0,1.0,1.0)
	
	    '  Load backdrop - from a .JPG because no alpha is required And its a large image.
	    Screen = LoadSprite("simplesoccer_res\\screen.x","simplesoccer_res\\grass.jpg",1.0,1.0)
	
	    '  Load title shadow image first - "Balcony Angels" font by Ray Larabie (www.fontfreak.com)
	    TitleShadow = LoadSprite("simplesoccer_res\\sprite.x","simplesoccer_res\\title_shadow.png",15.0,4.0)
	
	    '  Position title shadow slightly below And To the Right of the position where the title will be, To create a drop-shadow.
	    SetSpritePos(TitleShadow,ScreenCentreX + 0.4, 3.3 + 0.4)
	
	    '  Set the shadow's blend mode to multiply the destination by the color in the shadow image, i.e. where the shadow
	    '  image is light the background will show through in its natural color. Where its dark, the background will be
	    '  darkened To give the impression of shadow.
	    iSpriteRenderModeSet(TitleShadow,D3DBLEND_DESTCOLOR,D3DBLEND_ZERO)
	
	    '  Load title image.
	    Title = LoadSprite("simplesoccer_res\\sprite.x","simplesoccer_res\\title.png",15.0,4.0)
	
	    '  Position title shadow
	    SetSpritePos(Title,ScreenCentreX, 3.3)
	
	    '  Load controls image And position below title
	    Controls = LoadSprite("simplesoccer_res\\sprite.x","simplesoccer_res\\controls.png",8.0,2.0)
	    SetSpritePos(Controls,ScreenCentreX,8.2)
	
	    '  Initialise other objects
	    ball1.Initialise()
	    paddle1.Initialise()
	
	    '  Activate sprites at the End - where all sprite settings have been finalised
	    iSpriteShow(Screen)
	    iSpriteShow(TitleShadow)
	    iSpriteShow(Title)
	    iSpriteShow(Controls)
	    
	EndMethod

	Method _Run()
	   
	    '  Update paddle
	    ball1.Run()
	    paddle1.Run()
	
	EndMethod

EndType